local TagNode = class("TagNode")
ParseNode = class("ParseNode")
local BaseType = class("TagParse")

TagNode.BEGIN_TAG_TYPE = 1
TagNode.END_TAG_TYPE = 2
TagNode.SINGLE_TAG_TYPE = 3

function TagNode:ctor()
    self.Name = ""
    self.TagType = 0
    self.Attribute = {}
end

function TagNode:setName(name)
    if name~=nil and type(name)~="string" then
        return
    end

    self.Name = name
end

function TagNode:getName()
    return self.Name
end

function TagNode:setType(tagType)
    if tagType~=nil and type(tagType)~="number" then
        return
    end

    if tagType==nil then
        self.TagType = tagType
        return
    end

    if tagType<self.BEGIN_TAG_TYPE or tagType>self.SINGLE_TAG_TYPE then
        return
    end
    
    self.TagType = tagType
end

function TagNode:getType()
    return self.TagType
end

function TagNode:setAttribute(atrTable)
    if atrTable~=nil and type(atrTable)~="table" then
        return
    end

    self.Attribute = atrTable
end

function TagNode:getAttribute()
    return self.Attribute
end

function TagNode:showTag()
    print("TagNode.Name is "..self.Name)
    print("TagNode.TagType is "..self.TagType)
    print("TagNode.Attribute:")
    if self.Attribute~=nil and #self.Attribute~=0 then
        for k,v in ipairs(self.Attribute) do
            print("name is "..v[1]..", value is "..v[2])
        end
    end
    print("")
end

ParseNode.TAG_TYPE_DOUBLE = 1
ParseNode.TAG_TYPE_SINGLE = 2

function ParseNode:ctor()
    self.TagName = ""
    self.TagType = 0
    self.Attribute = {}
    self.Children = {}
end

function ParseNode:setTagName(tagName)
    if tagName~=nil and type(tagName)~="string" then
        return
    end

    self.TagName = tagName
end

function ParseNode:getTagName()
    return self.TagName
end

function ParseNode:setTagType(tagType)
    if tagType~=nil and type(tagType)~="number" then
        return
    end

    if tagType==nil then
        self.TagType = tagType
        return
    end

    if tagType<self.TAG_TYPE_DOUBLE or tagType>self.TAG_TYPE_SINGLE then
        return
    end

    self.TagType = tagType
end

function ParseNode:getTagType()
    return self.TagType
end

function ParseNode:setValue(value)
    if value~=nil and type(value)~="string" then
        return
    end

    self.Value = value
end

function ParseNode:getValue()
    return self.Value
end

function ParseNode:setAttribute(atrTable)
    if atrTable~=nil and type(atrTable)~="table" then
        return
    end

    self.Attribute = atrTable
end

function ParseNode:getAttribute()
    return self.Attribute
end

function ParseNode:setParent(parentNode)
    if parent~=nil and type(parentNode)~="table" then
        return
    end

    self.ParentNode = parentNode
end

function ParseNode:getParent()
    return self.ParentNode
end

function ParseNode:setLeftBrother(leftBrotherNode)
    if leftBrotherNode~=nil and type(leftBrotherNode)~="table" then
        return
    end

    self.LeftBrotherNode = leftBrotherNode
end

function ParseNode:getLeftBrother()
    return self.LeftBrotherNode
end

function ParseNode:setRightBrother(rightBrotherNode)
    if rightBrotherNode~=nil and type(rightBrotherNode)~="table" then
        return
    end

    self.RightBrotherNode = rightBrotherNode
end

function ParseNode:getRightBrother()
    return self.RightBrotherNode
end

function ParseNode:getChildrenSize()
    return #self.Children
end

function ParseNode:addChild(child)
    if type(child)~="table" then
        return
    end

    if #self.Children~=0 then
        local brother = self.Children[#self.Children]
        brother.RightBrotherNode = child
        child.LeftBrotherNode = brother
    end

    self.Children[#self.Children+1] = child
    child.ParentNode = self
end

function ParseNode:getChild()
    if #self.Children==0 then
        return nil
    end

    return self.Children[1]
end

function ParseNode:getChildByIndex(index)
    if index<1 or index>#self.Children then
        return nil
    end

    return self.Children[index]
end

function ParseNode:removeChild(child)
    if type(child)~="table" then
        return nil
    end

    if child.ParentNode~=self then
        return nil
    end

    child.ParentNode = nil

    local index = 0
    for k, v in ipairs(self.Children) do
        if v==child then
            index = k
            break
        end
    end

    if index==0 then
        return nil
    end

    table.remove(self.Children, index)

    local LeftBrother = child.LeftBrotherNode
    local RightBrother = child.RightBrotherNode
    if LeftBrother~=nil then
        child.LeftBrotherNode = nil
    end
    if RightBrother~=nil then
        child.RightBrotherNode = nil
    end
    if LeftBrother~=nil and RightBrother~=nil then
        LeftBrother.RightBrotherNode = RightBrother
        RightBrother.LeftBrotherNode = LeftBrother
    elseif LeftBrother~=nil then
        LeftBrother.RightBrotherNode = nil
    elseif RightBrother~=nil then
        RightBrother.LeftBrotherNode = nil
    end

    return child
end

function ParseNode:removeAllChildren()
    while #self.Children~=0 do
        local child = self.Children[#self.Children]

        child.ParentNode = nil

        local LeftBrother = child.LeftBrotherNode
        local RightBrother = child.RightBrotherNode
        if LeftBrother~=nil then
            child.LeftBrotherNode = nil
        end
        if RightBrother~=nil then
            child.RightBrotherNode = nil
        end
        if LeftBrother~=nil and RightBrother~=nil then
            LeftBrother.RightBrotherNode = RightBrother
            RightBrother.LeftBrotherNode = LeftBrother
        elseif LeftBrother~=nil then
            LeftBrother.RightBrotherNode = nil
        elseif RightBrother~=nil then
            RightBrother.LeftBrotherNode = nil
        end

        self.Children[#self.Children] = nil
    end
end

function ParseNode:removeChildByIndex(index)
    if index<1 or index>#self.Children then
        return nil
    end

    local child = self.Children[index]
    return self:removeChild(child)
end

function ParseNode:showNode()
    local Name = self.TagName
    local Value = self.Value
    if Name==nil then
        Name = "--"
    end
    if Value==nil then
        Value = "--"
    end
    print("ParseNode.TagName is "..Name)
    print("ParseNode.Value is "..Value)
    print("ParseNode.Children is "..#self.Children)
    print("ParseNode.Attribute:")
    if self.Attribute~=nil and #self.Attribute~=0 then
        for k,v in pairs(self.Attribute) do
            print("name is "..v[1]..", value is "..v[2])
        end
    end
    print("")
    for k,v in ipairs(self.Children) do
        v:showNode()
    end
end

function ParseNode:createStr()
    local Value = ""
    local BeginTagStr = ""
    local EndTagStr = ""
    local SingleTagStr = ""
    local ChildrenStr = ""
    local TagName = self.TagName

    local str = ""

    if string.len(TagName)~=0 then
        if self.TagType==self.TAG_TYPE_SINGLE then
            SingleTagStr = "<"..TagName.." />"
            str = SingleTagStr
        elseif self.TagType==self.TAG_TYPE_DOUBLE then
            local Attribute = ""
            if #self.Attribute~=0 then
                for k,v in ipairs(self.Attribute) do
                    local tempAttribute = " "..v[1].."=\""..v[2].."\""
                    Attribute = Attribute..tempAttribute
                end
            end
            BeginTagStr = "<"..TagName..Attribute..">"
            EndTagStr = "</"..TagName..">"

            for k,v in ipairs(self.Children) do
                local child = v
                ChildrenStr = ChildrenStr..child:createStr()
            end

            str = BeginTagStr..ChildrenStr..EndTagStr
        end
    else
        Value = self.Value
        str = Value
    end

    return str
end

--ͺ
function BaseType:parse(str)
    if type(str)~="string" then
        return nil
    end

    local tempStrTable = self:initStrTable(str)

    --for k,v in ipairs(tempStrTable) do
    --    print("~~~~~~str is "..v)
    --end

    if #tempStrTable==0 then
        return nil
    end

    local GrammarStack = {}

    local tempNodeTable = {}

    local NodeCountStack = {}
    local Count = 0

    while #tempStrTable~=0 do
        local tempStr = tempStrTable[1]
        if self:isTagStr(tempStr)==false then
            local tempNode = self:getNormalNodeByStr(tempStr)
            table.insert(tempNodeTable, tempNode)
            Count = Count + 1
            table.remove(tempStrTable, 1)
        elseif self:isSingleTagStr(tempStr)==true then
            local tempTag = self:getSingleTagNodeByStr(tempStr)
            local tempNode = self:getSingleNodeByNode(tempTag)
            table.insert(tempNodeTable, tempNode)
            Count = Count + 1
            table.remove(tempStrTable, 1)
        elseif self:isBeginTagStr(tempStr)==true then
            local tempTag = self:getBeginTagNodeByStr(tempStr)
            table.insert(GrammarStack, tempTag)
            table.insert(NodeCountStack, Count)
            Count = 0
            table.remove(tempStrTable, 1)
        elseif self:isEndTagStr(tempStr)==true then
            --˫ǩĽǩûпʼǩ
            if #GrammarStack==0 then
                return nil
            end

            local tempBeginTag = GrammarStack[#GrammarStack]
            local tempEndTag = self:getEndTagNodeByStr(tempStr)

            --˫ǩĿʼڽǩֲ
            if tempBeginTag:getName()~=tempEndTag:getName() then
                return nil
            end

            table.remove(GrammarStack)

            local tempNode = self:getDoubleNodeByNode(tempBeginTag)
            while Count~=0 do
                local lastCount = 0
                if #NodeCountStack~=0 then
                    lastCount = NodeCountStack[#NodeCountStack]
                end
                local child = tempNodeTable[lastCount+1]
                tempNode:addChild(child)
                table.remove(tempNodeTable, lastCount+1)
                Count = Count - 1
            end
            if #NodeCountStack~=0 then
                Count = NodeCountStack[#NodeCountStack]
                table.remove(NodeCountStack)
            end
            table.insert(tempNodeTable, tempNode)
            Count = Count + 1
            table.remove(tempStrTable, 1)
        end
    end

    if Count~=#tempNodeTable then
        print("Err->TagParse:parse(), Count~=#tempNodeTable!")
        return nil
    end



    if #tempNodeTable==0 then
        print("Err->TagParse:parse(), #tempNodeTable==0!")
        return nil
    end

    local Node = ParseNode.new()
    Node:setTagName("font")
    Node:setTagType(ParseNode.TAG_TYPE_DOUBLE)
    for k,v in ipairs(tempNodeTable) do
        Node:addChild(v)
    end

    return Node
end

--ַָСԪ
function BaseType:initStrTable(str)
    if type(str)~="string" then
        return {}
    end

    local StrTable = {}

    local LastStr = nil
    local TagStr = nil
    local RestStr = nil
    
    local function getSegmentationByTag(str)
        if type(str)~="string" then
            return nil
        end

        local LastStr = nil
        local TagStr = nil
        local RestStr = nil

        local format1 = "<"
        local format2 = ">"
        
        local BIndex = 0
        local EIndex = 0
        local length = string.len(str)

        BIndex = string.find(str, format1, BIndex+1)
        if BIndex==nil then
            LastStr=str
            return LastStr
        elseif BIndex>1 then
            LastStr = string.sub(str, 1, BIndex-1)
        end

        EIndex = string.find(str, format2, BIndex+1)
        if EIndex==nil then
            LastStr=str
            return LastStr
        elseif EIndex<length then
            RestStr = string.sub(str, EIndex+1, -1)
        end

        TagStr = string.sub(str, BIndex, EIndex)

        return LastStr, TagStr, RestStr
    end

    RestStr = str

    while RestStr~=nil and RestStr~="" do
        LastStr, TagStr, RestStr = getSegmentationByTag(RestStr)

        if LastStr~=nil and LastStr~="" then
            table.insert(StrTable, LastStr)
        end

        if TagStr~=nil and TagStr~="" then
            table.insert(StrTable, TagStr)
        end
    end

    return StrTable
end

--жǷΪǩ
function BaseType:isTagStr(str)
    if type(str)~="string" then
        return false
    end

    local format = "<.+>"
    local result = string.match(str, format)
    if result~=str then
        return false
    end

    local BIndex = 0
    local number = 0
    format = "<"
    while BIndex~=nil do
        BIndex = string.find(str, format, BIndex+1)
        if BIndex~=nil then
            number = number + 1
        end
    end
    if number~=1 then
        return false
    end

    BIndex = 0
    number = 0
    format = ">"
    while BIndex~=nil do
        BIndex = string.find(str, format, BIndex+1)
        if BIndex~=nil then
            number = number + 1
        end
    end
    if number~=1 then
        return false
    end

    return true
end

--жǷΪǩ
function BaseType:isSingleTagStr(str)
    if type(str)~="string" then
        return false
    end

    if self:isTagStr(str)~=true then
        return false
    end

    local format = "<%a+%s*/>"
    local result = string.match(str, format)
    if result==nil then
        return false
    end

    return true
end

--жǷΪ˫ǩĿʼǩ
function BaseType:isBeginTagStr(str)
    if type(str)~="string" then
        return false
    end

    if self:isTagStr(str)~=true then
        return false
    end

    local format = "<%a+"
    local result = string.match(str, format)
    if result==nil then
        return false
    end

    format = "/"
    result = string.match(str, format)
    if result~=nil then
        return false
    end

    return true
end

--жǷΪ˫ǩĽǩ
function BaseType:isEndTagStr(str)
    if type(str)~="string" then
        return false
    end

    if self:isTagStr(str)~=true then
        return false
    end

    format = "</%a+%s*>"
    result = string.match(str, format)
    if result==nil then
        return false
    end
    
    return true
end

--TagNodeǩ
function BaseType:getSingleTagNodeByStr(str)
    if type(str)~="string" then
        return nil
    end
    
    local format = "%a+"
    local name = string.match(str, format)
    if name==nil then
        return nil
    end

    local Node = TagNode.new()
    Node:setName(name)
    Node:setType(TagNode.SINGLE_TAG_TYPE)

    return Node
end

--TagNode˫ǩĿʼǩ
function BaseType:getBeginTagNodeByStr(str)
    if type(str)~="string" then
        return nil
    end
    
    local format = "%a+"
    local name = string.match(str, format)
    if name==nil then
        return nil
    end

    local Node = TagNode.new()
    Node:setName(name)
    Node:setType(TagNode.BEGIN_TAG_TYPE)

    local AttributeTable = {}

    format = "%a+=\"#?%w+\""
    local beginIndex = 0
    local endIndex = 0
--    print("str is "..str)
    while true do
        local tempStr = string.match(str, format, beginIndex+1)
        if tempStr==nil then
            break
        end
        beginIndex, endIndex = string.find(str, tempStr)
        beginIndex = endIndex

        local function getAttributeByStr(str)
            if type(str)~="string" then
                return nil
            end

            local format = "%a+"
            local name = string.match(str, format)

            format = "\""
            local beginIndex = string.find(str, format)
            local endIndex = string.find(str, format, beginIndex+1)
            local value = string.sub(str, beginIndex+1, endIndex-1)

            return name, value
        end

        local name, value = getAttributeByStr(tempStr)
        
        if string.len(value)>0 then
            local tempStr1 = string.match(value, "%s+")
            if tempStr1==nil then
                table.insert(AttributeTable, {name, value})
            end
        end
    end

    if #AttributeTable~=0 then
        Node:setAttribute(AttributeTable)
    end

    return Node
end

--TagNode˫ǩǩĽǩ
function BaseType:getEndTagNodeByStr(str)
    if type(str)~="string" then
        return nil
    end
    
    local format = "%a+"
    local name = string.match(str, format)
    if name==nil then
        return nil
    end

    local Node = TagNode.new()
    Node:setName(name)
    Node:setType(TagNode.END_TAG_TYPE)

    return Node
end

--ParseNodeǩParseNode
function BaseType:getSingleNodeByNode(tagNode)
    if type(tagNode)~="table" then
        return nil
    end

    if tagNode:getType()~=TagNode.SINGLE_TAG_TYPE then
        return nil
    end

    local Node = ParseNode.new()
    Node:setTagName(tagNode:getName())
    Node:setTagType(ParseNode.TAG_TYPE_SINGLE)

    return Node
end

--ParseNode˫ǩParseNode
function BaseType:getDoubleNodeByNode(tagNode)
    if type(tagNode)~="table" then
        return nil
    end

    if tagNode:getType()~=TagNode.BEGIN_TAG_TYPE then
        return nil
    end

    local Node = ParseNode.new()
    Node:setTagName(tagNode:getName())
    Node:setTagType(ParseNode.TAG_TYPE_DOUBLE)
    Node:setAttribute(tagNode:getAttribute())

    return Node
end

--ParseNodeͨParseNode
function BaseType:getNormalNodeByStr(str)
    if type(str)~="string" then
        return nil
    end

    local Node = ParseNode.new()
    Node:setValue(str)
    
    return Node
end

TagParse = TagParse or BaseType
